@@ -9,13 +9,15 @@ from paapi5_python_sdk.api.default_api import DefaultApi
 from paapi5_python_sdk.get_items_request import GetItemsRequest
 from paapi5_python_sdk.search_items_request import SearchItemsRequest
 from paapi5_python_sdk.get_variations_request import GetVariationsRequest
+from paapi5_python_sdk.get_browse_nodes_request import GetBrowseNodesRequest
 from paapi5_python_sdk.partner_type import PartnerType
 from paapi5_python_sdk.rest import ApiException
 
 from amazon.constant import DOMAINS, REGIONS, CONDITION
 from amazon.constant import PRODUCT_RESOURCES, SEARCH_RESOURCES, VARIATION_RESOURCES
+from amazon.constant import BROWSE_RESOURCES
 from amazon.exception import AmazonException
-from amazon.parse import parse_product
+from amazon.parse import parse_product, AmazonBrowseNode, parse_browsenode
 from amazon.tools import get_asin, chunks
 
 import time
@@ -162,8 +164,12 @@ class AmazonAPI:
             if len(check_product_id) > 1:
                 raise AmazonException('ValueError', 'Only 1 product ID is allowed, use '
                                                     'get_products for multiple requests')
-        return self.get_products(product_id, condition=condition, merchant=merchant,
-                                 async_req=async_req)[0]
+        product = self.get_products(product_id, condition=condition, merchant=merchant,
+                                    async_req=async_req)
+        if product:
+            return product[0]
+        else:
+            return None
 
     def search_products(self, item_count=10, item_page=1, items_per_page=10, keywords=None,
                         actor=None, artist=None, author=None, brand=None, title=None,
@@ -226,8 +232,8 @@ class AmazonAPI:
             raise AmazonException('ValueError', 'Arg items_per_page should be between 1 and 10')
         if item_count > 100 or item_count < 1:
             raise AmazonException('ValueError', 'Arg item_count should be between 1 and 100')
-        if item_page > 10 or item_page < 1:
-            raise AmazonException('ValueError', 'Arg item_page should be between 1 and 10')
+        if item_page < 1:
+            raise AmazonException('ValueError', 'Arg item_page should be 1 or higher')
         if not keywords and not actor and not artist and not author and not brand and not title:
             raise AmazonException('ValueError', 'At least one of the following args must be '
                                                 'provided: keywords, actor, artist, author, brand,'
@@ -284,15 +290,17 @@ class AmazonAPI:
                             results.append(parse_product(item))
                             if len(results) >= item_count:
                                 break
+                        if len(response.search_result.items) < items_per_page:
+                            break
                 else:
                     break
                 if response.errors is not None:
                     raise AmazonException(response.errors[0].code, response.errors[0].message)
             except Exception as e:
+                if e.status == "NoResults":
+                    break
                 raise AmazonException('ResponseError', e)
             item_page += 1
-            if item_page > 10:
-                break
 
         if results:
             return results
@@ -329,8 +337,8 @@ class AmazonAPI:
             raise AmazonException('ValueError', 'Arg items_per_page should be between 1 and 10')
         if item_count > 100 or item_count < 1:
             raise AmazonException('ValueError', 'Arg item_count should be between 1 and 100')
-        if item_page > 10 or item_page < 1:
-            raise AmazonException('ValueError', 'Arg item_page should be between 1 and 10')
+        if item_page < 1:
+            raise AmazonException('ValueError', 'Arg item_page should be 1 or higher')
 
         results = []
         while len(results) < item_count:
@@ -371,6 +379,8 @@ class AmazonAPI:
                             results.append(parse_product(item))
                             if len(results) >= item_count:
                                 break
+                        if len(response.variations_result.items) < items_per_page:
+                            break
                 else:
                     break
                 if response.errors is not None:
@@ -378,10 +388,61 @@ class AmazonAPI:
             except Exception as e:
                 raise AmazonException('ResponseError', e)
             item_page += 1
-            if item_page > 10:
-                break
 
         if results:
             return results
         else:
             return None
+
+    def get_browsenodes(self, browse_nodes, async_req=False):
+        """Get browse nodes information from Amazon.
+
+        Args:
+            browse_nodes (list): List of strings containing the browse node ids.
+            async_req (bool, optional): Specify if a thread should be created to
+                run the request. Defaults to False.
+
+        Returns:
+            dict: A dictionary containing the browse node information.
+        """
+
+        if isinstance(browse_nodes, list) is False:
+            raise Exception('Browse nodes parameter should be a list')
+        elif not browse_nodes:
+            raise Exception('Browse nodes parameter can\'t be empty')
+
+        try:
+            request = GetBrowseNodesRequest(
+                partner_tag=self.tag,
+                partner_type=PartnerType.ASSOCIATES,
+                marketplace=self.marketplace,
+                browse_node_ids=browse_nodes,
+                languages_of_preference=None,
+                resources=BROWSE_RESOURCES)
+        except ValueError as e:
+            raise AmazonException("ValueError", e)
+
+        try:
+            self._throttle()
+            if async_req:
+                thread = self.api.get_browse_nodes(request, async_req=True)
+                response = thread.get()
+            else:
+                response = self.api.get_browse_nodes(request)
+        except ApiException as e:
+            raise AmazonException('ApiException', e)
+
+        try:
+            if response.browse_nodes_result is not None:
+                res = [AmazonBrowseNode(item) for item in response.browse_nodes_result.browse_nodes]
+                return parse_browsenode(res)
+            if response.errors is not None:
+                raise AmazonException(response.errors[0].code, response.errors[0].message)
+        except TypeError as e:
+            raise AmazonException("TypeError", e)
+        except ValueError as e:
+            raise AmazonException(ValueError, e)
+        except AmazonException as e:
+            raise AmazonException(e.status, e.reason)
+        except Exception as e:
+            raise AmazonException("General", e)
